import openai


class Model:
    name = None
    edit_format = None
    max_context_tokens = 0
    tokenizer = None

    always_available = False
    use_repo_map = False
    send_undo_reply = False

    prompt_price = None
    completion_price = None

    @classmethod
    def create(cls, name):
        from .openai import OpenAIModel
        from .openrouter import OpenRouterModel

        if "openrouter.ai" in openai.api_base:
            return OpenRouterModel(name)
        return OpenAIModel(name)

    def __str__(self):
        return self.name

    @staticmethod
    def strong_model():
        return Model.create("gpt-4")

    @staticmethod
    def weak_model():
        return Model.create("gpt-3.5-turbo")

    @staticmethod
    def commit_message_models():
        return [Model.create("gpt-3.5-turbo"), Model.create("gpt-3.5-turbo-16k")]



# flake8: noqa: E501

from .base_prompts import CoderPrompts


class EditBlockPrompts(CoderPrompts):
    main_system = """Act as an expert software developer.
Always use best practices when coding.
When you edit or add code, respect and use existing conventions, libraries, etc.

Take requests for changes to the supplied code.
If the request is ambiguous, ask questions.

Once you understand the request you MUST:
1. List the files you need to modify. *NEVER* suggest changes to a *read-only* file. Instead, you *MUST* tell the user their full path names and ask them to *add the files to the chat*. End your reply and wait for their approval.
2. Think step-by-step and explain the needed changes.
3. Describe each change with a *SEARCH/REPLACE block* per the example below.
"""

    system_reminder = """You MUST use a *SEARCH/REPLACE block* to modify the source file:

{fence[0]}python
some/dir/example.py
<<<<<<< SEARCH
    # Multiplication function
    def multiply(a,b)
        "multiply 2 numbers"

        return a*b
=======
    # Addition function
    def add(a,b):
        "add 2 numbers"

        return a+b
>>>>>>> REPLACE
{fence[1]}

The *SEARCH* section must *EXACTLY MATCH* the existing source code, character for character.

Every *SEARCH/REPLACE block* must be fenced with {fence[0]} and {fence[1]}, with the correct code language.

Every *SEARCH/REPLACE block* must start with the full path!
NEVER try to *SEARCH/REPLACE* any *read-only* files.

If you want to put code in a new file, use a *SEARCH/REPLACE block* with:
- A new file path, including dir name if needed
- An empty `SEARCH` section
- The new file's contents in the `updated` section
"""

    files_content_prefix = "These are the *read-write* files:\n"

    files_no_full_files = "I am not sharing any *read-write* files yet."

    repo_content_prefix = """Below here are summaries of other files!
Do not propose changes to these files, they are *read-only*.
To make a file *read-write*, ask me to *add it to the chat*.
"""
